# Live at Rhizom

## About

This archive contains code and support files for a live performance at RHIZOM 2022 under the pseudonym Zuei Podoxa.

[More information](https://zuei.ch/rhizom)

## Installing

### Requirements

To run the code in this repository, you need

- a recent version of [SuperCollider 3](https://supercollider.github.io/) (> 3.11.2)
- a recent version of [sc3-plugins](https://github.com/supercollider/sc3-plugins)
- a specific set of community contributed packages (["Quarks"](https://github.com/supercollider-quarks/quarks)) for SuperCollider - see the lists in the `quark_list` folder (`required_quarks.txt`)

### Installing the library

These instructions assume that you have installed and set up SC3, sc3-plugins and all of the required Quarks (using `Quarks.install`).

Copy all of the folders to a directory of your choice. Take note of the directory path.

Install the zuei extension (see folder contents below) by copying the `/extension/zuei` folder to your [user (or system) extensions directory](https://doc.sccode.org/Guides/UsingExtensions.html). Open `zueilib.scd`, search for `ncUserDir` and insert the installation path noted earlier.

Include the following lines in your [sclang start-up file](https://doc.sccode.org/Reference/StartupFile.html) to fix some common error messages when compiling large SynthDefs:

```sclang
s.options.numBuffers = 2.pow(18);   // increase this if you need to load more samples
s.options.memSize = 2.pow(20);      // increase this if you get "alloc failed" messages
s.options.numWireBufs = 2.pow(16);  // increase this if you get "exceeded number of interconnect buffers" messages
s.options.maxNodes = 2.pow(16);     // increase this if you are getting dropouts and "too many nodes" messages
```

To load all of the necessary library code (including environment variables and `ProxySpace` settings) execute the following lines within your sclang session:

```sclang
s.waitForBoot {
    (Platform.userExtensionDir +/+ "zuei" +/+ "zueilib.scd").load;
    s.sync;
    Require(z.ncUserDir +/+ "lib" +/+ "main");
};
```

## Folder contents

- `/extension` - "Zuei" class extensions and main system pseudoclass
- `/performance` - Patches used during the live performance
- `/quark_list` - List of quarks required to run the SuperCollider code files and script to export list of currently installed quarks (refspecs/names)
- `/rider` - Performance tech rider as provided to the festival organizers
- `/util` - Various utility scripts used during development

## License

The SuperCollider code (`.scd` files) is released under the [ISC license](https://www.isc.org/licenses/).
