+ AbstractFunction {
	bpmcps { ^this.composeUnaryOp('bpmcps') }
	cpsbpm { ^this.composeUnaryOp('cpsbpm') }
	bpmdur { ^this.composeUnaryOp('bpmdur') }
	durbpm { ^this.composeUnaryOp('durbpm') }

	cpsmel { ^this.composeUnaryOp('cpsmel') }
	melcps { ^this.composeUnaryOp('melcps') }

	taupole { ^this.composeUnaryOp('taupole') }
	poletau { ^this.composeUnaryOp('poletau') }

	decayfb { ^this.composeUnaryOp('decayfb') }
	fbdecay { ^this.composeUnaryOp('fbdecay') }

}

+ SequenceableCollection {
	bpmcps { ^this.performUnaryOp('bpmcps') }
	cpsbpm { ^this.performUnaryOp('cpsbpm') }
	bpmdur { ^this.performUnaryOp('bpmdur') }
	durbpm { ^this.performUnaryOp('durbpm') }

	cpsmel { ^this.performUnaryOp('cpsmel') }
	melcps { ^this.performUnaryOp('melcps') }

	taupole { ^this.performUnaryOp('taupole') }
	poletau { ^this.performUnaryOp('poletau') }

	decayfb { ^this.performUnaryOp('decayfb') }
	fbdecay { ^this.performUnaryOp('fbdecay') }
}

+ SimpleNumber {
	bpmcps { ^this / 60.0 }
	cpsbpm { ^this * 60.0 }
	bpmdur { ^60 / this }
	durbpm { ^60 / this }

	cpsmel { arg formula=\OShaughnessy80;
		switch (formula)
		{ \OShaughnessy80 } { ^2595 * log10(1 + (this/700)) }
		{ \Fant1968 } { ^(1000/log(2)) * log(1 + (this/1000)) }
	}

	melcps { arg formula=\OShaughnessy80;
		switch (formula)
		{ \OShaughnessy80 } { ^	700 * (10.pow(this/2595) - 1) }
		{ \Fant1968 } { ^1000 * (2.pow(this/1000) - 1) }
	}

	taupole { arg sampleRate=44100;
		var sr = Server.default.sampleRate ?? sampleRate;

		if (this.isStrictlyPositive.not) {
			"% - Value not strictly positive.".format(this.name).error;
		};
		^exp(-1.0/(this*sr))
	}

	poletau { arg sampleRate=44100;
		var sr = Server.default.sampleRate ?? sampleRate;

		if (this.isStrictlyPositive.not or: { this > 1 }) {
			"% - Value not in range (0,1).".format(this.name).error;
		};

		^-1.0 / (log(this) * sr)
	}

	decayfb { arg delayTime;
		^0.001 ** (delayTime / this.abs) * this.sign
	}

	fbdecay { arg delayTime;
		^delayTime * log(1000) / log(this.reciprocal)
	}
}

+ Symbol {
	bpmcps { ^this }
	cpsbpm { ^this }
	bpmdur { ^this }
	durbpm { ^this }

	cpsmel { ^this }
	melcps { ^this }

	taupole { ^this }
	poletau { ^this }

	decayfb { ^this }
	fbdecay { ^this }
}

+ UGen {
	bpmcps { ^this / 60.0 }
	cpsbpm { ^this * 60.0 }
	bpmdur { ^60 / this }
	durbpm { ^60 / this }

	cpsmel { arg formula=\OShaughnessy80;
		switch (formula)
		{ \OShaughnessy80 } { ^2595 * log10(1 + (this/700)) }
		{ \Fant1968 } { ^(1000/log(2)) * log(1 + (this/1000)) }
	}

	melcps { arg formula=\OShaughnessy80;
		switch (formula)
		{ \OShaughnessy80 } { ^	700 * (10.pow(this/2595) - 1) }
		{ \Fant1968 } { ^1000 * (2.pow(this/1000) - 1) }
	}

	taupole { arg sampleRate=44100;
		var sr = Server.default.sampleRate ?? sampleRate;
		^exp(-1.0/(this*sr))
	}

	poletau { arg sampleRate=44100;
		var sr = Server.default.sampleRate ?? sampleRate;
		^-1.0 / (log(this) * sr)
	}

	decayfb { arg delayTime;
		^0.001 ** (delayTime / this.abs) * this.sign
	}

	fbdecay { arg delayTime;
		^delayTime * log(1000) / log(this.reciprocal)
	}
}
