/*
zueilib

flexible pseudoclass using object modeling
contains common helpers
*/

z = ();
z.reg = { |self, name, implementation, strict=true, verbose=0|
	var symbolicname = name.asSymbol;
	if (self.respondsTo(symbolicname)) {
		var txt = "\\" ++ symbolicname + "is a reserved method name (already defined by Event or its superclasses).";
		if (strict) {
			Exception(txt).throw;
		} {
			txt.warn;
		}
	} {
		if (self.keys.includes(symbolicname)) {
			var txt = "\\" ++ symbolicname + "already registered.";
			if (strict) {
				Exception(txt).throw;
			} {
				txt.warn;
			}
		} {
			if (verbose != 0) {
				("Registered" + "\\" ++ symbolicname ++ ".").postln;
			};
			self[symbolicname] = implementation;
		};
	};
};

z.reg(\speakerTest, { |self, volume=(-20)|
	if (Server.default.serverRunning) {
		{ PinkNoise.ar * volume.dbamp ! 2 }.play;
	} {
		"Server is not running or fully ready yet.".warn;
	};
});

z.reg(\ncUserDir, { |self|
	// IMPORTANT: insert correct paths to parent directory of live performance folder here
	Platform.case(
		\osx,       { }, // insert macOS path here
		\linux,     { }, // insert linux path here
		\windows,   { }  // insert windows path here
	);
}.value);

z.reg(\testBufMono, Buffer.read(s, Platform.resourceDir +/+ "sounds/a11wlk01.wav"));
z.reg(\testBufStereo, Buffer.read(s, z.ncUserDir ++ "example_audio/storm.wav"));
z.reg(\testBufEtenraku, Buffer.read(s, z.ncUserDir ++ "example_audio/etenraku.wav"));

z.reg(\clockPollTask, Task({
	{
		var pollRate = 10;
		[~lhpclock.bar % 4, ~lhpclock.beatInBar.round(1e-3), ~lhpclock.timeToNextBeat(quant: -4).round(1e-3)].postln;
		pollRate.reciprocal.wait;
		// 0.1.wait;
	}.loop;
}));

"zueilib loaded!"
