// functions for creating randomized envelopes for synthesis

~randEnv = { |numSegs, dur=1|
	var env, levels, times, curves;

	levels = { rrand(-1.0, 1.0) } ! (numSegs+1);
	times = { exprand(1, 10) } ! numSegs;
	curves = { [rrand(-4.0, 4.0) , \sin, \wel].wchoose([20,7,1].normalizeSum) } ! numSegs;
	env = Env(levels, times, curves);
	env
};


~randCircleEnv = { |numSegs, dur=1, circle=false, bipolar=true, initialLevel=nil, curveRange=4.0, timeLo=1.0, timeHi=10.0, verbose=false, saveLastToEnv=false|
	var env, levels, times, curves;

	circle.postln;

	if (circle) {
		if (initialLevel.isNil) {
			levels = { rrand(-1.0, 1.0) } ! numSegs;
			levels = levels ++ levels[0];
		} {
			if (bipolar) {
				levels = { rrand(-1.0, 1.0) } ! (numSegs-1);
				levels = [initialLevel] ++ levels ++ [initialLevel];
			} {
				levels = { rrand(-1.0, 1.0) } ! (numSegs-1);
				levels = [initialLevel.linlin(0,1,-1,1)] ++ levels ++ [initialLevel.linlin(0,1,-1,1)];
			}
		}
	} {
		levels = { rrand(-1.0, 1.0) } ! (numSegs+1);
	};
	levels = levels.normalize(bipolar.asInteger.neg, 1.0);

	times = { exprand(timeLo, timeHi) } ! numSegs;
	times = times.normalizeSum * dur;

	curveRange = curveRange.asFloat;
	// curves = { curveRange.rand2 } ! numSegs;
	curves = Array.fill(numSegs, { |i|
		if (i%2 == 0) { curveRange.rand } { curveRange.rand.neg };
	});

	env = Env(levels, times, curves);

	if (verbose) {
		"levels: %\ntimes: %\ncurves: %\n".postf(levels.round(1e-2), times.round(1e-2), curves.round(1e-2));
	};

	if (saveLastToEnv) {
		~lastRandCircleEnv = env;
	};

	env
};

~randAscendingEnv = { |numSegs, dur=1, randCurve|
	var env, times, curves;
	var levels = List.new;

	randCurve = randCurve ?? { rrand(-20,20) };

	levels.add(-1.0);
	numSegs.do{ |i|
		var lastVal = levels[i];
		var randRange = 2/(numSegs-1)/2;

		levels.add(lastVal + rrand(0, randRange).lincurve(0, randRange, 0, randRange, randCurve))
	};
	levels = levels.linlin(-1.0, levels.maxItem, rrand(-1, -0.1), rrand(0.5,1));

	times = { exprand(1, 10) } ! numSegs;
	curves = { [rrand(-4.0, 4.0) , \sin, \wel].wchoose([20,7,1].normalizeSum) } ! numSegs;
	env = Env(levels, times, curves);
	env
};

"Random envelope generator functions loaded".postln;