// add a global/main "safety net" limiter to the sound output to prevent harsh clipping distortion
// prints warning to stdout when peaks exceed 0dB

// requires the SafetyNet and SafetyLimiter (https://github.com/nhthn/supercollider-safety-limiter) quarks
(
Safety.addSynthDefFunc(\safeLimitNotify, { |numChans, limit=1|
    {
        var limitCtl = \limit.kr(limit).abs;
        var mainOuts = In.ar(0, numChans);
        var safeOuts = ReplaceBadValues.ar(mainOuts);
        var resetTrig = Impulse.ar(0.1);
        var peak = Peak.ar(safeOuts * (1 - resetTrig), resetTrig);
        var maxPeak = peak.reduce(\max);
        var delayMaxPeak = Delay1.ar(maxPeak);
        var overTrig = delayMaxPeak > limitCtl;
        var limited = SafetyLimiter.ar(safeOuts);
        (delayMaxPeak / limitCtl).ampdb.poll((delayMaxPeak > maxPeak) * overTrig, "Audio peak exceeded limit by dB");
        ReplaceOut.ar(0, limited);
    }
});

Safety.defaultDefName = \safeLimitNotify;
)
