/*
utility for generating Ndef template strings for live coding
*/


// print available arguments for all installed UGen classes
(
var listCollector, classes;

listCollector = { |node, list|
	list.add(node);
	node.subclasses.do{ |n| listCollector.value(n, list) };
};

classes = List.new;
listCollector.value(UGen, classes);

classes = classes.reject{ |c|
	c.class.findMethod('ar').argNames.reject{ |i| [\this, \mul, \add].includes(i) }.size == 0
};
classes = classes.reject{ |c|
	["HOA", "Foa", "encode", "decode", "Stk"].collect(c.name.asString.contains(_)).reduce(\or)
};
classes = classes.sort{ |a, b| a.name < b.name };

classes.do { |c|
	var args, longestItemLen;
	args = c.class.findMethod('ar').argNames.reject{ |i| [\this, \mul, \add].includes(i) };
	longestItemLen = classes.collect{ |c| c.asString.size }.maxItem;

	(c.asString.padRight(longestItemLen + 2) ++ args.asString).postln;
	// "%\t\t\t %\n".postf(c.asString, args);
};
)


// create ndef template for a given class
// includes default specs for all arguments (using JITLibExtension's .addSpec method)
(
var template, makeTemplate, exampleClass, exampleArgs;

template = "Ndef(%).clear;
Ndef(%).ar(2);
(
Ndef(%).addSpec(
%
);
Ndef(%, { |%|
    var sig;

    sig = %.ar(%);

    sig ! 2 * -20.dbamp
}).play;
)
";

makeTemplate = { |class, args|
	var specsString, specArgs, argsString, defArgsString, longestArgLen, classNameString;
	var defaultArgVals, specDefaultArgVals;

	longestArgLen = args.collect{ |a| a.asString.size }.maxItem;
	defaultArgVals = class.ar.inputs;

	specsString = "";

	specArgs = args.copy;
	specDefaultArgVals = defaultArgVals.copy;
	if (specArgs.includesAny([\input, \in, \freq])) {
		var indicesToRemove;
		indicesToRemove = [\input, \in, \freq].collect{ |i| specArgs.indexOf(i) };
		indicesToRemove.select(_.notNil).do{ |i| specDefaultArgVals.removeAt(i)};

		specArgs = specArgs.reject{ |i| [\input, \in, \freq].includes(i) };
	};

	specArgs.do{ |i, it|
		var argName, newLine, defaultSpec;

		argName = "\\" ++ (i.asString ++ ",").padRight(longestArgLen + 2);
		newLine = if (it == specArgs.lastIndex) { "" } { ",\n" };

		defaultSpec = "[0, 1, \\lin, 0, %]".format(specDefaultArgVals[it]);

		specsString = specsString ++ "    %%%".format(argName, defaultSpec, newLine).padLeft(4);
	};

	argsString = args.asString;
	["[", "]", " "].do{ |ch|
		argsString = argsString.replace(ch, "");
	};
	argsString = argsString.replace(",", ", ");


	defArgsString = "";
	args.do{ |i, it|
		var argSeparator;
		argSeparator = if (it == args.lastIndex) { "" } { ", " };

		defArgsString = defArgsString ++ "%=%%".format(i, defaultArgVals[it], argSeparator);
	};

	classNameString = class.asString;
	classNameString = "\\" ++ classNameString[0].toLower ++ classNameString[1..];

	template.format(
		classNameString,
		classNameString,
		classNameString,
		specsString,
		classNameString,
		defArgsString,
		class.asString,
		argsString
	);
};


exampleClass = Compander;
exampleArgs = exampleClass.class.findMethod('ar').argNames.reject{ |i| [\this, \mul, \add].includes(i) };

template = makeTemplate.value(exampleClass, exampleArgs);
template.newTextWindow(exampleClass.asString);
);
